/*
Copyright (c) 2008-2009 Bryan Head
All Rights Reserved

[This software is released under the "MIT License"]

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the
Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall
be included in all copies or substantial portions of the
Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package org.headb.sandpile;

import javax.media.opengl.GLAutoDrawable;
import java.util.List;
import gnu.trove.list.array.TIntArrayList;
import java.util.concurrent.locks.Lock;

/**
 * The interface for classes that dictate how a sandpile is displayed on a
 * canvas.
 * @author Bryan Head
 */
public interface SandpileDrawer {

    public enum ColorMode {

        NUM_OF_GRAINS,
        STABILITY,
        FIRINGS,
        DIFFERENCE
    }

    public GLAutoDrawable getCanvas();

    public void setColorMode(ColorMode cm);

    public void setColors(Float2dArrayList colors, Float2dArrayList inDebtColors, float[] backgorundColor);

    public ColorMode getColorMode();

    public void paintSandpileGraph(SandpileGraph graph, Float2dArrayList vertexLocations, SandpileConfiguration config, TIntArrayList firings, TIntArrayList selectedVertices, Lock configLock);

    public void setBaseConfig(SandpileConfiguration baseConfig);

    public void setSelectionBox(float maxX, float maxY, float minX, float minY);

    public void clearSelectionBox();

    public float[] transformCanvasCoords(int x, int y);

    public float getZoom();
}
